/***********************************************************************/
/*
  raster_file.c

  Revised by Mark Lake, 28/07/20017, for r.skyline in GRASS 7.x
  Revised by Mark Lake, 26/07/20017, for r.horizon in GRASS 7.x
  Revised by Mark Lake, 16/07/2007, for r.horizon in GRASS 6.x
  Written by Mark Lake, 15/07/2002, for r.horizon in GRASS 5.x

*/
/***********************************************************************/


#include <math.h>
#include "raster_file.h"


/***********************************************************************/
/* Public functions                                                    */
/***********************************************************************/

void Close_raster_file (int map_fd)
{
  Rast_close (map_fd);
}


/***********************************************************************/

void Copy_raster_row_buf_to_binary_CELL_buf (void* in_buf, void* out_buf, 
					     RASTER_MAP_TYPE in_type)
{
  int col, ncols;
  CELL cval;
  DCELL dval;
  void *in_buf_ptr, *out_buf_ptr;

  ncols = Rast_window_cols();

  /* Read data cell by cell */

  in_buf_ptr = in_buf;
  out_buf_ptr = out_buf;
  for (col = 0; col < ncols; col ++)
    {
      if (in_type == CELL_TYPE)
	{
	  cval = Rast_get_c_value (in_buf_ptr, in_type);
	  if (! (Rast_is_null_value(&cval, CELL_TYPE)))
	    {
	      if (cval != 0)
		cval = 1;
	      else 
		cval = 0;
	    }
	  else
	    {
	      cval = 0;
	    }
	  
	  Rast_set_c_value (out_buf_ptr, cval, CELL_TYPE);
	}
      else
	{
	  dval = Rast_get_d_value (in_buf_ptr, in_type);
 	  if (! (Rast_is_null_value(&dval, DCELL_TYPE)))
	    {
	      if (dval != 0)
		cval = 1;
	      else 
		cval = 0;
	    }
	  else
	    {
	      cval = 0;
	    }

	  Rast_set_c_value (out_buf_ptr, cval, CELL_TYPE);
	}

      in_buf_ptr = G_incr_void_ptr (in_buf_ptr,
				    Rast_cell_size(in_type));
      out_buf_ptr = G_incr_void_ptr (out_buf_ptr,
				     Rast_cell_size(CELL_TYPE));	    
    }
}


/***********************************************************************/

int Open_raster_infile (char* map_name, char* mapset, char* search_mapset,
			RASTER_MAP_TYPE* map_type, char* message)
{
  int map_fd;

  if (G_find_raster (map_name, search_mapset) == NULL)
    {
      /* G_fatal_error (_("Can't find input map ")); */
      sprintf (message, "Can't find %s for input ", map_name);
      return -1;
    }

  strcpy (mapset, G_find_raster (map_name, search_mapset));
  if ((map_fd = Rast_open_old (map_name, mapset)) < 0)
    {
      /* G_fatal_error (_("Can't open  input map ")); */
      sprintf (message, "Can't open %s for input ", map_name);
      return map_fd;
    }
    
  *map_type = Rast_map_type(map_name, mapset);
  return map_fd;
}



